<?php

namespace Drupal\Tests\commerce_vado\Functional;

use Drupal\Tests\commerce_cart\Functional\CartBrowserTestBase;
use Drupal\Core\Entity\Entity\EntityViewDisplay;
use Drupal\views\Views;

/**
 * Tests commerce_vado cart features.
 *
 * @group commerce_vado
 */
class VadoCartTest extends CartBrowserTestBase {

  /**
   * The order storage.
   *
   * @var \Drupal\commerce_order\OrderStorage
   */
  protected $orderStorage;

  /**
   * Modules to enable.
   *
   * @var array
   */
  public static $modules = [
    'commerce_vado',
  ];

  /**
   * {@inheritdoc}
   */
  protected function getAdministratorPermissions() {
    return array_merge([
      'access vado administration pages',
    ], parent::getAdministratorPermissions());
  }

  /**
   * {@inheritdoc}
   */
  protected function setUp(): void {
    parent::setUp();
    /** @var \Drupal\commerce_vado\VadoFieldManagerInterface $vado_field_manager */
    $vado_field_manager = \Drupal::service('commerce_vado.field_manager');
    /** @var \Drupal\Core\Entity\EntityFieldManagerInterface $entity_field_manager */
    $this->entity_field_manager = \Drupal::service('entity_field.manager');

    $vado_field_manager->installField('child_variations', 'default');
    $vado_field_manager->installField('sync_quantity', 'default');
    // Test that the actual fields now exist on the variation type.
    $this->assertTrue($vado_field_manager->hasField('child_variations', 'default'));
    $this->assertTrue($vado_field_manager->hasField('sync_quantity', 'default'));

    $this->orderStorage = $this->container->get('entity_type.manager')->getStorage('commerce_order');

    // Mke sure the view display for the product is set to combine.
    $product_view_display = EntityViewDisplay::load('commerce_product.default.default');
    if (!$product_view_display) {
      $product_view_display = EntityViewDisplay::create([
        'targetEntityType' => 'commerce_product',
        'bundle' => 'default',
        'mode' => 'default',
        'status' => TRUE,
      ]);
    }
    $product_view_display->setComponent('variations', [
      'type' => 'commerce_add_to_cart',
      'region' => 'content',
      'label' => 'hidden',
      'settings' => [
        'combine' => TRUE,
      ],
    ]);
    $product_view_display->save();

    /** @var \Drupal\commerce_product\Entity\Product $parent_product */
    $this->parent_product = $this->createEntity('commerce_product', [
      'type' => 'default',
      'title' => 'Parent Product',
      'stores' => [$this->store],
      'body' => ['value' => 'This is a parent variation'],
      'variations' => [
        $this->createEntity('commerce_product_variation', [
          'type' => 'default',
          'sku' => 'PARENT',
          'price' => [
            'number' => '0',
            'currency_code' => 'USD',
          ],
        ]),
      ],
    ]);
    /** @var \Drupal\commerce_product\Entity\Product $child_product */
    $this->child_product = $this->createEntity('commerce_product', [
      'type' => 'default',
      'title' => 'Child Product',
      'stores' => [$this->store],
      'body' => ['value' => 'This is a child variation'],
      'variations' => [
        $this->createEntity('commerce_product_variation', [
          'type' => 'default',
          'sku' => 'CHILD',
          'price' => [
            'number' => '75.99',
            'currency_code' => 'USD',
          ],
        ]),
      ],
    ]);
    $this->child_product_1 = $this->createEntity('commerce_product', [
      'type' => 'default',
      'title' => 'Child Product 1',
      'stores' => [$this->store],
      'body' => ['value' => 'This is a child variation'],
      'variations' => [
        $this->createEntity('commerce_product_variation', [
          'type' => 'default',
          'sku' => 'CHILD-1',
          'price' => [
            'number' => '55.99',
            'currency_code' => 'USD',
          ],
        ]),
      ],
    ]);
    /** @var \Drupal\commerce_product\Entity\ProductVariation $this->parent_variation */
    $this->parent_variation = $this->parent_product->getDefaultVariation();
    /** @var \Drupal\commerce_product\Entity\ProductVariation $chid_product_variation */
    $this->child_variation = $this->child_product->getDefaultVariation();
    $this->child_variation_1 = $this->child_product_1->getDefaultVariation();

    $this->child_ids = [
      $this->child_variation->id(),
      $this->child_variation_1->id(),
      $this->child_variation_1->id(),
    ];
  }

  /**
   * Tests VADO without quantity sync.
   *
   * @throws \Drupal\Core\Entity\EntityStorageException
   */
  public function testSyncQuantityOff() {
    // Add parent to cart before setting child.
    $this->drupalGet($this->parent_product->toUrl());
    $this->submitForm([], 'Add to cart');
    $this->orderStorage->resetCache([$this->cart->id()]);
    $this->cart = $this->orderStorage->load($this->cart->id());
    $this->assertCount(1, $this->cart->getItems());
    // Test the cart page quantity exists.
    $this->drupalGet('cart');
    $this->assertSession()->fieldValueEquals('edit-edit-quantity-0', 1);
    $this->cartManager->emptyCart($this->cart);

    // Attach the children to the parent and ensure sync quantity is off.
    $this->parent_variation->set('child_variations', $this->child_ids);
    $this->parent_variation->set('sync_quantity', FALSE);
    $this->parent_variation->save();
    // Add parent to cart after setting child.
    $this->drupalGet($this->parent_product->toUrl());
    $this->submitForm([], 'Add to cart');
    $this->orderStorage->resetCache([$this->cart->id()]);
    $this->cart = $this->orderStorage->load($this->cart->id());
    // Note get items does not reflect total quantity, but order items.
    $this->assertCount(3, $this->cart->getItems());
    // Test the cart page.
    $this->drupalGet('cart');
    $this->assertSession()->fieldValueEquals('edit-edit-quantity-0', 1);
    $this->assertSession()->fieldValueEquals('edit-edit-quantity-1', 1);
    $this->assertSession()->fieldValueEquals('edit-edit-quantity-2', 2);
    $this->assertSession()->buttonExists('edit-remove-button-0');
    $this->assertSession()->buttonExists('edit-remove-button-1');
    $this->assertSession()->buttonExists('edit-remove-button-2');
    $this->assertSession()->pageTextContains('Parent Product');
    $this->assertSession()->pageTextContains('Child Product');
    $this->assertSession()->pageTextContains('Child Product 1');

    // Edit the parent quantity to make sure the child does NOT sync.
    $values = [
      'edit_quantity[0]' => 2,
    ];
    $this->assertSession()->buttonExists('Update cart');
    $this->submitForm($values, t('Update cart'));
    $this->assertSession()->fieldValueEquals('edit-edit-quantity-0', 2);
    $this->assertSession()->fieldValueEquals('edit-edit-quantity-1', 1);
    $this->assertSession()->fieldValueEquals('edit-edit-quantity-2', 2);

    // Remove the parent and make sure it's child stays in the cart.
    $values = [
      'edit_quantity[0]' => 0,
    ];
    $this->submitForm($values, t('Update cart'));
    $this->assertSession()->fieldValueEquals('edit-edit-quantity-0', 1);
    $this->assertSession()->fieldValueEquals('edit-edit-quantity-1', 2);
    $this->assertSession()->fieldNotExists('edit-edit-quantity-2');
    $this->assertSession()->pageTextNotContains('Parent Product');
    $this->assertSession()->pageTextContains('Child Product');
    $this->assertSession()->pageTextContains('Child Product 1');
    $this->assertSession()->pageTextContains(t('Your shopping cart has been updated.'));
  }

  /**
   * Tests VADO with quantity sync.
   *
   * @throws \Drupal\Core\Entity\EntityStorageException
   */
  public function testSyncQuantityOn() {
    // Add parent to cart before setting child.
    $this->drupalGet($this->parent_product->toUrl());
    $this->submitForm([], 'Add to cart');
    $this->orderStorage->resetCache([$this->cart->id()]);
    $this->cart = $this->orderStorage->load($this->cart->id());
    $this->assertCount(1, $this->cart->getItems());
    // Test the cart page quantity exists.
    $this->drupalGet('cart');
    $this->assertSession()->fieldValueEquals('edit-edit-quantity-0', 1);
    $this->cartManager->emptyCart($this->cart);

    // Attach the child to the parent and ensure sync quantity is on.
    $this->parent_variation->set('child_variations', $this->child_ids);
    $this->parent_variation->set('sync_quantity', TRUE);
    $this->parent_variation->save();
    // Add parent to cart after setting child.
    $this->drupalGet($this->parent_product->toUrl());
    $this->submitForm([], 'Add to cart');
    $this->orderStorage->resetCache([$this->cart->id()]);
    $this->cart = $this->orderStorage->load($this->cart->id());
    // Note get items does not reflect total quantity, but order items...
    $this->assertCount(3, $this->cart->getItems());
    // Test the cart page.
    $this->drupalGet('cart');
    $this->assertSession()->fieldValueEquals('edit-edit-quantity-0', 1);
    $this->assertSession()->fieldValueEquals('edit-edit-quantity-1', 1);
    $this->assertSession()->fieldValueEquals('edit-edit-quantity-2', 2);
    $this->assertSession()->buttonExists('edit-remove-button-0');

    // Test that the remove button is removed for the synced variation.
    $this->assertSession()->buttonNotExists('edit-remove-button-1');
    // Test that the quanity widget is disabled for synced the variation.
    $this->assertSession()->elementAttributeContains('css', 'input#edit-edit-quantity-1', 'disabled', 'disabled');
    $this->assertSession()->elementAttributeContains('css', 'input#edit-edit-quantity-2', 'disabled', 'disabled');
    // Test that the Remove button on the parent has changed to "Remove Bundle".
    $this->assertSession()->buttonExists('Remove Bundle');

    // Add the child by itself to make sure it doesn't combine.
    // This should create a new line item in the cart.
    $this->drupalGet($this->child_product->toUrl());
    $this->submitForm([], 'Add to cart');
    $this->orderStorage->resetCache([$this->cart->id()]);
    $this->cart = $this->orderStorage->load($this->cart->id());
    // Note get items does not reflect total quantity, but order items.
    $this->assertCount(4, $this->cart->getItems());
    // Test the cart page.
    $this->drupalGet('cart');
    $this->assertSession()->fieldValueEquals('edit-edit-quantity-0', 1);
    $this->assertSession()->fieldValueEquals('edit-edit-quantity-1', 1);
    $this->assertSession()->fieldValueEquals('edit-edit-quantity-2', 2);
    $this->assertSession()->fieldValueEquals('edit-edit-quantity-3', 1);

    // Add parent to cart again to test quantities. This is different
    // than updating as it calls on the cartEntityAddEvent again.
    $this->drupalGet($this->parent_product->toUrl());
    $this->submitForm([], 'Add to cart');
    $this->orderStorage->resetCache([$this->cart->id()]);
    $this->cart = $this->orderStorage->load($this->cart->id());
    $this->assertCount(4, $this->cart->getItems());
    $this->drupalGet('cart');
    $this->assertSession()->fieldValueEquals('edit-edit-quantity-0', 2);
    $this->assertSession()->fieldValueEquals('edit-edit-quantity-1', 2);
    $this->assertSession()->fieldValueEquals('edit-edit-quantity-2', 4);
    $this->assertSession()->fieldValueEquals('edit-edit-quantity-3', 1);

    // Update the parent quantity to make sure the child syncs.
    $values = [
      'edit_quantity[0]' => 3,
    ];
    $this->assertSession()->buttonExists('Update cart');
    $this->submitForm($values, t('Update cart'));
    $this->assertSession()->fieldValueEquals('edit-edit-quantity-0', 3);
    $this->assertSession()->fieldValueEquals('edit-edit-quantity-1', 3);
    $this->assertSession()->fieldValueEquals('edit-edit-quantity-2', 6);
    $this->assertSession()->fieldValueEquals('edit-edit-quantity-3', 1);
    $values = [
      'edit_quantity[0]' => 1,
    ];
    $this->submitForm($values, t('Update cart'));
    $this->assertSession()->fieldValueEquals('edit-edit-quantity-0', 1);
    $this->assertSession()->fieldValueEquals('edit-edit-quantity-1', 1);
    $this->assertSession()->fieldValueEquals('edit-edit-quantity-2', 2);
    $this->assertSession()->fieldValueEquals('edit-edit-quantity-3', 1);
    $this->assertSession()->elementTextContains('css', 'td.views-field.views-field-unit-price__number', '$0.00');
    $this->assertSession()->pageTextContains('$0.00');
    $this->assertSession()->pageTextContains('$75.99');
    $this->assertSession()->pageTextContains('$55.99');

    // Enable hide parent zero.
    $this->drupalGet('/admin/commerce/config/vado');
    $this->assertSession()->checkboxNotChecked('edit-settings-container-hide-parent-zero-price');
    $enable = [
      'settings_container[hide_parent_zero_price]' => TRUE,
    ];
    $this->submitForm($enable, 'Save');
    $this->assertSession()->checkboxChecked('edit-settings-container-hide-parent-zero-price');
    // We have to clear the view cache for hide parent zero.
    $view = Views::getView('commerce_cart_form');
    $view->storage->invalidateCaches();
    $this->drupalGet('cart');
    $this->assertSession()->fieldValueEquals('edit-edit-quantity-0', 1);
    $this->assertSession()->fieldValueEquals('edit-edit-quantity-1', 1);
    $this->assertSession()->fieldValueEquals('edit-edit-quantity-2', 2);
    $this->assertSession()->fieldValueEquals('edit-edit-quantity-3', 1);
    $this->assertSession()->elementTextNotContains('css', 'td.views-field.views-field-unit-price__number', '$0.00');
    $this->assertSession()->pageTextContains('$75.99');
    $this->assertSession()->pageTextContains('$55.99');

    // Remove the parent and make sure it's child is also removed.
    $values = [
      'edit_quantity[0]' => 0,
    ];
    $this->submitForm($values, t('Update cart'));
    $this->assertSession()->fieldValueEquals('edit-edit-quantity-0', 1);
    $this->assertSession()->fieldNotExists('edit-edit-quantity-1');
    $this->assertSession()->fieldNotExists('edit-edit-quantity-2');
    $this->assertSession()->fieldNotExists('edit-edit-quantity-3');

    // Remove the child that was added without the parent.
    $values = [
      'edit_quantity[0]' => 0,
    ];
    $this->submitForm($values, t('Update cart'));
    $this->assertSession()->pageTextContains(t('Your shopping cart is empty.'));
  }

}
